@@ -36,8 +36,7 @@ static void doubleBoxSize(QPDFObjectHandle& page, char const* box_name)
     {
 	doubled.push_back(
 	    QPDFObjectHandle::newReal(
-		QUtil::double_to_string(
-		    box.getArrayItem(i).getNumericValue() * 2.0, 2)));
+                box.getArrayItem(i).getNumericValue() * 2.0, 2));
     }
     page.replaceKey(box_name, QPDFObjectHandle::newArray(doubled));
 }
@@ -52,6 +51,15 @@ int main(int argc, char* argv[])
 	whoami += 3;
     }
 
+    // For test suite
+    bool static_id = false;
+    if ((argc > 1) && (strcmp(argv[1], " --static-id") == 0))
+    {
+        static_id = true;
+        --argc;
+        ++argv;
+    }
+
     if (! ((argc == 3) || (argc == 4)))
     {
 	usage();
@@ -62,12 +70,7 @@ int main(int argc, char* argv[])
     char const* password = (argc == 4) ? argv[3] : "";
 
     // Text to prepend to each page's contents
-    char const* content = "2 0 0 2 0 0 cm\n";
-
-    // Copy text into a buffer without the null terminator
-    PointerHolder<Buffer> b = new Buffer(strlen(content));
-    unsigned char* bp = b->getBuffer();
-    memcpy(bp, (unsigned char*)content, strlen(content));
+    std::string content = "2 0 0 2 0 0 cm\n";
 
     try
     {
@@ -81,7 +84,8 @@ int main(int argc, char* argv[])
 	    QPDFObjectHandle& page = *iter;
 
 	    // Prepend the buffer to the page's contents
-	    page.addPageContents(QPDFObjectHandle::newStream(&qpdf, b), true);
+	    page.addPageContents(
+                QPDFObjectHandle::newStream(&qpdf, content), true);
 
 	    // Double the size of each of the content boxes
 	    doubleBoxSize(page, "/MediaBox");
@@ -93,7 +97,7 @@ int main(int argc, char* argv[])
 
 	// Write out a new file
 	QPDFWriter w(qpdf, outfilename);
-	if (QUtil::get_env("IN_TESTSUITE"))
+	if (static_id)
 	{
 	    // For the test suite, uncompress streams and use static
 	    // IDs.
