@@ -9,7 +9,9 @@ Pl_ASCIIHexDecoder::Pl_ASCIIHexDecoder(char const* identifier, Pipeline* next) :
     pos(0),
     eod(false)
 {
-    strcpy(this->inbuf, "00");
+    this->inbuf[0] = '0';
+    this->inbuf[1] = '0';
+    this->inbuf[2] = '\0';
 }
 
 Pl_ASCIIHexDecoder::~Pl_ASCIIHexDecoder()
@@ -91,14 +93,16 @@ Pl_ASCIIHexDecoder::flush()
 	    b[i] = this->inbuf[i] - '0';
 	}
     }
-    unsigned char ch = (unsigned char)((b[0] << 4) + b[1]);
+    unsigned char ch = static_cast<unsigned char>((b[0] << 4) + b[1]);
 
     QTC::TC("libtests", "Pl_ASCIIHexDecoder partial flush",
 	    (this->pos == 2) ? 0 : 1);
     getNext()->write(&ch, 1);
 
     this->pos = 0;
-    strcpy(this->inbuf, "00");
+    this->inbuf[0] = '0';
+    this->inbuf[1] = '0';
+    this->inbuf[2] = '\0';
 }
 
 void
