@@ -3,10 +3,15 @@
 
 #include <qpdf/QUtil.hh>
 #include <qpdf/PointerHolder.hh>
+#ifdef USE_INSECURE_RANDOM
+# include <qpdf/InsecureRandomDataProvider.hh>
+#endif
+#include <qpdf/SecureRandomDataProvider.hh>
 
 #include <cmath>
 #include <iomanip>
 #include <sstream>
+#include <stdexcept>
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
@@ -200,7 +205,7 @@ QUtil::hex_encode(std::string const& input)
     for (unsigned int i = 0; i < input.length(); ++i)
     {
         result += QUtil::int_to_string_base(
-            static_cast<int>(static_cast<unsigned char>(input[i])), 16, 2);
+            static_cast<int>(static_cast<unsigned char>(input.at(i))), 16, 2);
     }
     return result;
 }
@@ -232,13 +237,9 @@ QUtil::setLineBuf(FILE* f)
 char*
 QUtil::getWhoami(char* argv0)
 {
-#ifdef _WIN32
-    char pathsep = '\\';
-#else
-    char pathsep = '/';
-#endif
     char* whoami = 0;
-    if ((whoami = strrchr(argv0, pathsep)) == NULL)
+    if (((whoami = strrchr(argv0, '/')) == NULL) &&
+        ((whoami = strrchr(argv0, '\\')) == NULL))
     {
 	whoami = argv0;
     }
@@ -246,13 +247,13 @@ QUtil::getWhoami(char* argv0)
     {
 	++whoami;
     }
-#ifdef _WIN32
+
     if ((strlen(whoami) > 4) &&
 	(strcmp(whoami + strlen(whoami) - 4, ".exe") == 0))
     {
 	whoami[strlen(whoami) - 4] = '\0';
     }
-#endif
+
     return whoami;
 }
 
@@ -261,6 +262,9 @@ QUtil::get_env(std::string const& var, std::string* value)
 {
     // This was basically ripped out of wxWindows.
 #ifdef _WIN32
+# ifdef NO_GET_ENVIRONMENT
+    return false;
+# else
     // first get the size of the buffer
     DWORD len = ::GetEnvironmentVariable(var.c_str(), NULL, 0);
     if (len == 0)
@@ -278,6 +282,7 @@ QUtil::get_env(std::string const& var, std::string* value)
     }
 
     return true;
+# endif
 #else
     char* p = getenv(var.c_str());
     if (p == 0)
@@ -360,11 +365,11 @@ QUtil::toUTF8(unsigned long uval)
 	    // Maximum that will fit in high byte now shrinks by one bit
 	    maxval >>= 1;
 	    // Slide to the left one byte
-	    --cur_byte;
-	    if (cur_byte < bytes)
+	    if (cur_byte <= bytes)
 	    {
 		throw std::logic_error("QUtil::toUTF8: overflow error");
 	    }
+	    --cur_byte;
 	}
 	// If maxval is k bits long, the high (7 - k) bits of the
 	// resulting byte must be high.
@@ -377,24 +382,69 @@ QUtil::toUTF8(unsigned long uval)
     return result;
 }
 
+// Random data support
+
 long
 QUtil::random()
 {
-    static bool seeded_random = false;
-    if (! seeded_random)
-    {
-	// Seed the random number generator with something simple, but
-	// just to be interesting, don't use the unmodified current
-	// time.  It would be better if this were a more secure seed.
-        QUtil::srandom(QUtil::get_current_time() ^ 0xcccc);
-	seeded_random = true;
-    }
+    long result = 0L;
+    initializeWithRandomBytes(
+        reinterpret_cast<unsigned char*>(&result),
+        sizeof(result));
+    return result;
+}
 
-#ifdef HAVE_RANDOM
-    return ::random();
+static RandomDataProvider* random_data_provider = 0;
+
+#ifdef USE_INSECURE_RANDOM
+static RandomDataProvider* insecure_random_data_provider =
+    InsecureRandomDataProvider::getInstance();
 #else
-    return rand();
+static RandomDataProvider* insecure_random_data_provider = 0;
 #endif
+static RandomDataProvider* secure_random_data_provider =
+    SecureRandomDataProvider::getInstance();
+
+static void
+initialize_random_data_provider()
+{
+    if (random_data_provider == 0)
+    {
+        if (secure_random_data_provider)
+        {
+            random_data_provider = secure_random_data_provider;
+        }
+        else if (insecure_random_data_provider)
+        {
+            random_data_provider = insecure_random_data_provider;
+        }
+    }
+    // QUtil.hh has comments indicating that getRandomDataProvider(),
+    // which calls this method, never returns null.
+    if (random_data_provider == 0)
+    {
+        throw std::logic_error("QPDF has no random data provider");
+    }
+}
+
+void
+QUtil::setRandomDataProvider(RandomDataProvider* p)
+{
+    random_data_provider = p;
+}
+
+RandomDataProvider*
+QUtil::getRandomDataProvider()
+{
+    initialize_random_data_provider();
+    return random_data_provider;
+}
+
+void
+QUtil::initializeWithRandomBytes(unsigned char* data, size_t len)
+{
+    initialize_random_data_provider();
+    random_data_provider->provideRandomData(data, len);
 }
 
 void
@@ -406,12 +456,3 @@ QUtil::srandom(unsigned int seed)
     srand(seed);
 #endif
 }
-
-void
-QUtil::initializeWithRandomBytes(unsigned char* data, size_t len)
-{
-    for (size_t i = 0; i < len; ++i)
-    {
-        data[i] = static_cast<unsigned char>((QUtil::random() & 0xff0) >> 4);
-    }
-}
